@@ -14,9 +14,18 @@
 #   limitations under the License.
 
 __author__ = 'Michael Gruber'
-__version__ = '0.1.7'
+__version__ = '0.1.8'
+
+try:
+    from importlib import import_module
+except ImportError as import_error:
+    print(str(import_error))
+    print('fluentmock does not define importlib as a dependency,')
+    print('because importlib is part of the standard library')
+    print('starting with Python version 2.7')
+    print('')
+    print('Please install importlib using "pip install importlib".')
 
-from importlib import import_module
 from mock import Mock, call, patch
 from logging import getLogger
 from unittest import TestCase
@@ -25,26 +34,27 @@ from types import ModuleType
 LOGGER = getLogger(__name__)
 
 
-MESSAGE_COULD_NOT_VERIFY = 'Could not verify {call_entry}'
 MESSAGE_HAS_BEEN_CALLED_AT_LEAST_ONCE = """{call_entry} should NEVER have been called,
 but has been called at least once."""
-MESSAGE_INVALID_ATTRIBUTE = 'The target "{target_name}" has no attribute called "{attribute_name}".'
 MESSAGE_NO_CALLS = """
 Expected: {expected}
  but was: no patched function has been called.
 """
-MESSAGE_EXPECTED_BUT_WAS = """
-Expected: {expected}
- but was: {actual}
-"""
 
 NEVER = 0
 
 _configurators = {}
-_patches = []
+_patch_entries = []
 _call_entries = []
 
 
+class FluentAnyArguments(object):
+    pass
+
+
+ANY_ARGUMENTS = FluentAnyArguments()
+
+
 class UnitTests(TestCase):
 
     def setUp(self):
@@ -63,14 +73,7 @@ class UnitTests(TestCase):
         pass
 
 
-class FluentAnyArguments(object):
-    pass
-
-
-ANY_ARGUMENTS = FluentAnyArguments()
-
-
-class FluentTargeting(object):
+class FluentTarget(object):
 
     def __init__(self, target):
         if isinstance(target, ModuleType):
@@ -84,8 +87,21 @@ class FluentTargeting(object):
 
 class FluentAnswer(object):
 
-    RETURN_ANSWER = 0
-    RAISE_ANSWER = 1
+    class AnswerByReturning(object):
+
+        def __init__(self, value):
+            self._value = value
+
+        def __call__(self):
+            return self._value
+
+    class AnswerByRaising(object):
+
+        def __init__(self, value):
+            self._value = value
+
+        def __call__(self):
+            raise self._value
 
     def __init__(self, arguments, kwargs):
         self.arguments = arguments
@@ -97,40 +113,30 @@ class FluentAnswer(object):
             return None
 
         if len(self._answers) == 1:
-            value = self._answers[0]
+            answer = self._answers[0]
 
         if len(self._answers) > 1:
-            value = self._answers.pop(0)
+            answer = self._answers.pop(0)
 
-        return self.give_answer(value)
+        return answer()
 
-    def give_answer(self, answer):
-        answer_type = answer[1]
-        answer_value = answer[0]
-        if answer_type == self.RETURN_ANSWER:
-            return answer_value
-
-        if answer_type == self.RAISE_ANSWER:
-            raise answer_value
-
-        raise NotImplementedError('The answer type %s is not implemented')
+    def append_answer(self, answer_class, value):
+        answer = answer_class(value)
+        self._answers.append(answer)
 
     def then_return(self, value):
-        self._answers.append((value, self.RETURN_ANSWER))
+        self.append_answer(self.AnswerByReturning, value)
         return self
 
     def then_raise(self, value):
-        self._answers.append((value, self.RAISE_ANSWER))
+        self.append_answer(self.AnswerByRaising, value)
         return self
 
-    def __repr__(self):
-        return "Answer(arguments={arguments}, values={values})".format(arguments=self.arguments, values=self._answers)
 
-
-class FluentPatchEntry(FluentTargeting):
+class FluentPatchEntry(FluentTarget):
 
     def __init__(self, target, attribute_name, original):
-        FluentTargeting.__init__(self, target)
+        FluentTarget.__init__(self, target)
         self._attribute_name = attribute_name
         self._original = original
         self._patch = None
@@ -151,9 +157,10 @@ class FluentPatchEntry(FluentTargeting):
             self._patch.__exit__()
 
 
-class FluentCallEntry(FluentTargeting):
+class FluentCallEntry(FluentTarget):
+
     def __init__(self, target, attribute, arguments, keyword_arguments):
-        FluentTargeting.__init__(self, target)
+        FluentTarget.__init__(self, target)
         self._attribute_name = attribute
         self._arguments = arguments
         self._keyword_arguments = keyword_arguments
@@ -166,23 +173,16 @@ class FluentCallEntry(FluentTargeting):
         return False
 
     def __repr__(self):
-        arguments_as_strings = []
-        for argument in self._arguments:
-            if type(argument) == str:
-                arguments_as_strings.append("'{argument}'".format(argument=argument))
-            else:
-                arguments_as_strings.append(str(argument))
-
-        call_string = str(call(*self._arguments, **self._keyword_arguments)).replace('call', '')
-        return 'call {target_name}.{attribute_name}{call_string}'.format(target_name=self._target_name,
-                                                                         attribute_name=self._attribute_name,
-                                                                         call_string=call_string)
+        target_string = 'call {target_name}.{attribute_name}'.format(target_name=self._target_name,
+                                                                     attribute_name=self._attribute_name)
+        call_string = str(call(*self._arguments, **self._keyword_arguments))
+        return call_string.replace('call', target_string)
 
 
-class FluentMock(FluentTargeting):
+class FluentMock(FluentTarget):
 
     def __init__(self, target, attribute_name):
-        FluentTargeting.__init__(self, target)
+        FluentTarget.__init__(self, target)
         self._attribute_name = attribute_name
         self._answers = []
 
@@ -190,13 +190,10 @@ class FluentMock(FluentTargeting):
         call_entry = FluentCallEntry(self._target, self._attribute_name, arguments, keyword_arguments)
         _call_entries.append(call_entry)
 
-        if not self._answers:
-            return None
-
         for answer in self._answers:
             if answer.arguments == arguments and answer.kwargs == keyword_arguments:
                 return answer.next()
-            if answer.arguments and answer.arguments[0] == ANY_ARGUMENTS:
+            if answer.arguments and answer.arguments[0] is ANY_ARGUMENTS:
                 return answer.next()
 
         return None
@@ -204,9 +201,6 @@ class FluentMock(FluentTargeting):
     def append_new_answer(self, answer):
         self._answers.append(answer)
 
-    def __str__(self):
-        return "Mock(" + str(self._answers) + ")"
-
 
 class FluentMockConfigurator(object):
 
@@ -214,48 +208,83 @@ class FluentMockConfigurator(object):
         self._mock = mock
         self._arguments = None
         self._answer = None
-        self._kwargs = None
+        self._keyword_arguments = None
 
-    def __call__(self, *arguments, **kwargs):
+    def __call__(self, *arguments, **keyword_arguments):
         self._arguments = arguments
-        self._kwargs = kwargs
-        self._answer = FluentAnswer(self._arguments, self._kwargs)
+        self._keyword_arguments = keyword_arguments
+        self._answer = FluentAnswer(self._arguments, self._keyword_arguments)
         self._mock.append_new_answer(self._answer)
         return self._answer
 
 
-class FluentWhen(FluentTargeting):
+class FluentWhen(FluentTarget):
 
     def __init__(self, target):
-        FluentTargeting.__init__(self, target)
+        FluentTarget.__init__(self, target)
 
     def _get_original_attribute(self, name):
 
         if not hasattr(self._target, name):
-            error_message = MESSAGE_INVALID_ATTRIBUTE.format(target_name=self._target_name, attribute_name=name)
-            raise FluentMockException(error_message)
+            raise InvalidAttributeError(self._target_name, name)
 
         return getattr(self._target, name)
 
     def __getattr__(self, name):
         original = self._get_original_attribute(name)
         patch_entry = FluentPatchEntry(self._target, name, original)
-        _patches.append(patch_entry)
+        _patch_entries.append(patch_entry)
 
-        key = (self._target, name)
-        if not key in _configurators:
+        configurator_key = (self._target, name)
+        if not configurator_key in _configurators:
             fluent_mock = FluentMock(self._target, name)
             mock_configurator = FluentMockConfigurator(fluent_mock)
             patch_entry.patch_away_with(fluent_mock)
-            _configurators[key] = mock_configurator
+            _configurators[configurator_key] = mock_configurator
 
-        return _configurators[key]
+        return _configurators[configurator_key]
 
 
-class Verifier(FluentTargeting):
+class InvalidAttributeError(Exception):
+
+    MESSAGE_FORMAT = 'The target "{target_name}" has no attribute called "{attribute_name}".'
+
+    def __init__(self, target_name, attribute_name):
+        error_message = self.MESSAGE_FORMAT.format(target_name=target_name, attribute_name=attribute_name)
+        super(InvalidAttributeError, self).__init__(error_message)
+
+
+class CouldNotVerifyCallError(AssertionError):
+
+    MESSAGE_FORMAT = 'Could not verify {expected}'
+
+    def __init__(self, expected_call_entry):
+        error_message = self.MESSAGE_FORMAT.format(expected=expected_call_entry)
+        super(CouldNotVerifyCallError, self).__init__(error_message)
+
+
+class TargetHasBeenCalledWithDifferentArguments(AssertionError):
+
+    MESSAGE_FORMAT = """
+Expected: {expected}
+ but was: {actual}
+"""
+    ADDITIONAL_CALL_ENTRIES = '          {actual}\n'
+
+    def __init__(self, expected_call_entry, found_calls):
+
+        error_message = self.MESSAGE_FORMAT.format(expected=expected_call_entry, actual=found_calls[0])
+        if len(found_calls) > 1:
+            for call_entry in found_calls[1:]:
+                error_message += self.ADDITIONAL_CALL_ENTRIES.format(actual=call_entry)
+
+        super(TargetHasBeenCalledWithDifferentArguments, self).__init__(error_message)
+
+
+class Verifier(FluentTarget):
 
     def __init__(self, target, times):
-        FluentTargeting.__init__(self, target)
+        FluentTarget.__init__(self, target)
         self._attribute_name = None
         self._times = times
 
@@ -266,59 +295,47 @@ class Verifier(FluentTargeting):
         self._attribute_name = name
 
         if not hasattr(self._target, name):
-            raise FluentMockException(self.format_message(MESSAGE_INVALID_ATTRIBUTE))
+            raise InvalidAttributeError(self._target_name, name)
 
         return self
 
+    def __call__(self, *arguments, **keyword_arguments):
+        if self._times == 0:
+            self._assert_never_called(*arguments, **keyword_arguments)
+        else:
+            self._assert_called(*arguments, **keyword_arguments)
+
+    def _assert_never_called(self, *arguments, **keyword_arguments):
+        for call_entry in _call_entries:
+            if call_entry.verify(self._target, self._attribute_name, arguments, keyword_arguments):
+                error_message = MESSAGE_HAS_BEEN_CALLED_AT_LEAST_ONCE.format(call_entry=call_entry)
+                raise AssertionError(error_message)
+
     def _assert_called(self, *arguments, **keyword_arguments):
+        expected_call_entry = FluentCallEntry(self._target, self._attribute_name, arguments, keyword_arguments)
+
         if not _call_entries:
-            call_entry = FluentCallEntry(self._target, self._attribute_name, arguments, keyword_arguments)
-            error_message = MESSAGE_NO_CALLS.format(expected=call_entry)
-            raise AssertionError(error_message)
+            raise AssertionError(MESSAGE_NO_CALLS.format(expected=expected_call_entry))
 
         for call_entry in _call_entries:
             if call_entry.verify(self._target, self._attribute_name, arguments, keyword_arguments):
                 return
 
+        found_calls = self._find_calls_to_same_target()
+
+        if found_calls:
+            raise TargetHasBeenCalledWithDifferentArguments(expected_call_entry, found_calls)
+
+        raise CouldNotVerifyCallError(expected_call_entry)
+
+    def _find_calls_to_same_target(self):
         found_calls = []
 
         for call_entry in _call_entries:
             if call_entry._target == self._target and call_entry._attribute_name == self._attribute_name:
                 found_calls.append(call_entry)
 
-        number_of_found_calls = len(found_calls)
-        if number_of_found_calls > 0:
-            expected_call_entry = FluentCallEntry(self._target, self._attribute_name, arguments, keyword_arguments)
-            error_message = MESSAGE_EXPECTED_BUT_WAS.format(expected=expected_call_entry, actual=found_calls[0])
-            if number_of_found_calls > 1:
-                for call_entry in found_calls[1:]:
-                    error_message += '          {call_entry}\n'.format(call_entry=call_entry)
-            raise AssertionError(error_message)
-
-        call_entry = FluentCallEntry(self._target, self._attribute_name, arguments, keyword_arguments)
-        error_message = MESSAGE_COULD_NOT_VERIFY.format(call_entry=call_entry)
-        raise AssertionError(error_message)
-
-    def __call__(self, *arguments, **keyword_arguments):
-        if self._times == 0:
-            if not _call_entries:
-                return
-
-            for call_entry in _call_entries:
-                if call_entry.verify(self._target, self._attribute_name, arguments, keyword_arguments):
-                    error_message = MESSAGE_HAS_BEEN_CALLED_AT_LEAST_ONCE.format(call_entry=call_entry)
-                    raise AssertionError(error_message)
-
-            return
-        else:
-            self._assert_called(*arguments, **keyword_arguments)
-
-    def format_message(self, message, arguments='()'):
-        return message.format(target_name=self._target_name, attribute_name=self._attribute_name, arguments=arguments)
-
-
-class FluentMockException(Exception):
-    pass
+        return found_calls
 
 
 def when(target):
@@ -326,18 +343,18 @@ def when(target):
 
 
 def undo_patches():
-    global _call_entries, _patches, _configurators
+    global _call_entries, _patch_entries, _configurators
 
-    for _patch in _patches:
-        _patch.undo()
+    for patch_entry in _patch_entries:
+        patch_entry.undo()
 
     _call_entries = []
-    _patches = []
+    _patch_entries = []
     _configurators = {}
 
 
 def get_patches():
-    return _patches
+    return _patch_entries
 
 
 def verify(target, times=1):
